package sun.awt.windows;

import java.security.AccessController;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.security.PrivilegedAction;
import java.awt.image.BufferedImage;
import sun.java2d.pipe.Region;
import java.awt.peer.ComponentPeer;
import java.awt.dnd.DropTarget;
import java.awt.Component;
import java.awt.Font;
import java.awt.Color;
import java.awt.Event;
import java.awt.Window;
import java.util.List;
import sun.awt.SunToolkit;
import sun.awt.AWTAccessor;
import java.util.Iterator;
import java.awt.Dialog;
import java.io.File;
import java.awt.FileDialog;
import java.util.Vector;
import java.io.FilenameFilter;
import java.awt.peer.FileDialogPeer;

final class WFileDialogPeer extends WWindowPeer implements FileDialogPeer
{
    private WComponentPeer parent;
    private FilenameFilter fileFilter;
    private Vector<WWindowPeer> blockedWindows;
    
    private static native void setFilterString(String p0);
    
    @Override
    public void setFilenameFilter(FilenameFilter fileFilter) {
        this.fileFilter = fileFilter;
    }
    
    boolean checkFilenameFilter(String pathname) {
        FileDialog fileDialog = (FileDialog)this.target;
        if (this.fileFilter == null) {
            return true;
        }
        File file = new File(pathname);
        return this.fileFilter.accept(new File(file.getParent()), file.getName());
    }
    
    WFileDialogPeer(FileDialog fileDialog) {
        super(fileDialog);
        this.blockedWindows = new Vector();
    }
    
    @Override
    void create(WComponentPeer parent) {
        this.parent = parent;
    }
    
    @Override
    protected void checkCreation() {
    }
    
    @Override
    void initialize() {
        this.setFilenameFilter(((FileDialog)this.target).getFilenameFilter());
    }
    
    private native void _dispose();
    
    @Override
    protected void disposeImpl() {
        WToolkit.targetDisposedPeer(this.target, this);
        this._dispose();
    }
    
    private native void _show();
    
    private native void _hide();
    
    @Override
    public void show() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                WFileDialogPeer.access$000(WFileDialogPeer.this);
            }
        }).start();
    }
    
    @Override
    void hide() {
        this._hide();
    }
    
    void setHWnd(long hwnd) {
        if (this.hwnd == hwnd) {
            return;
        }
        this.hwnd = hwnd;
        for (WWindowPeer wWindowPeer : this.blockedWindows) {
            if (hwnd != 0L) {
                wWindowPeer.modalDisable((Dialog)this.target, hwnd);
            }
            else {
                wWindowPeer.modalEnable((Dialog)this.target);
            }
        }
        return;
    }
    
    void handleSelected(char[] value) {
        String[] split = new String(value).split("\u0000");
        String s;
        String substring;
        File[] array;
        if (split.length > 1) {
            s = split[0];
            int n = split.length - 1;
            array = new File[n];
            for (int i = 0; i < n; ++i) {
                array[i] = new File(s, split[i + 1]);
            }
            substring = split[1];
        }
        else {
            int lastIndex = split[0].lastIndexOf((int)File.separatorChar);
            if (lastIndex == -1) {
                s = "." + File.separator;
                substring = split[0];
            }
            else {
                s = split[0].substring(0, lastIndex + 1);
                substring = split[0].substring(lastIndex + 1);
            }
            array = new File[] { new File(s, substring) };
        }
        final FileDialog target = (FileDialog)this.target;
        AWTAccessor.FileDialogAccessor fileDialogAccessor = AWTAccessor.getFileDialogAccessor();
        fileDialogAccessor.setDirectory(target, s);
        fileDialogAccessor.setFile(target, substring);
        fileDialogAccessor.setFiles(target, array);
        SunToolkit.executeOnEventHandlerThread(target, new Runnable() {
            @Override
            public void run() {
                target.setVisible(false);
            }
        });
    }
    
    void handleCancel() {
        final FileDialog target = (FileDialog)this.target;
        AWTAccessor.getFileDialogAccessor().setFile(target, null);
        AWTAccessor.getFileDialogAccessor().setFiles(target, (File[])null);
        AWTAccessor.getFileDialogAccessor().setDirectory(target, null);
        SunToolkit.executeOnEventHandlerThread(target, new Runnable() {
            @Override
            public void run() {
                target.setVisible(false);
            }
        });
    }
    
    void blockWindow(WWindowPeer e) {
        this.blockedWindows.add(e);
        if (this.hwnd != 0L) {
            e.modalDisable((Dialog)this.target, this.hwnd);
        }
    }
    
    void unblockWindow(WWindowPeer o) {
        this.blockedWindows.remove(o);
        if (this.hwnd != 0L) {
            o.modalEnable((Dialog)this.target);
        }
    }
    
    @Override
    public void blockWindows(List<Window> list) {
        Iterator<Object> iterator = list.iterator();
        while (iterator.hasNext()) {
            WWindowPeer wWindowPeer = (WWindowPeer)AWTAccessor.getComponentAccessor().getPeer((Window)iterator.next());
            if (wWindowPeer != null) {
                this.blockWindow(wWindowPeer);
            }
            else {
                continue;
            }
        }
    }
    
    @Override
    public native void toFront();
    
    @Override
    public native void toBack();
    
    @Override
    public void updateAlwaysOnTopState() {
    }
    
    @Override
    public void setDirectory(String s) {
    }
    
    @Override
    public void setFile(String s) {
    }
    
    @Override
    public void setTitle(String s) {
    }
    
    @Override
    public void setResizable(boolean b) {
    }
    
    @Override
    void enable() {
    }
    
    @Override
    void disable() {
    }
    
    @Override
    public void reshape(int n, int n2, int n3, int n4) {
    }
    
    public boolean handleEvent(Event event) {
        return false;
    }
    
    @Override
    public void setForeground(Color color) {
    }
    
    @Override
    public void setBackground(Color color) {
    }
    
    @Override
    public void setFont(Font font) {
    }
    
    @Override
    public void updateMinimumSize() {
    }
    
    @Override
    public void updateIconImages() {
    }
    
    public boolean requestFocus(boolean b, boolean b2) {
        return false;
    }
    
    @Override
    public boolean requestFocus(Component component, boolean b, boolean b2, long n, CausedFocusEvent.Cause cause) {
        return false;
    }
    
    @Override
    void start() {
    }
    
    @Override
    public void beginValidate() {
    }
    
    @Override
    public void endValidate() {
    }
    
    void invalidate(int n, int n2, int n3, int n4) {
    }
    
    @Override
    public void addDropTarget(DropTarget dropTarget) {
    }
    
    @Override
    public void removeDropTarget(DropTarget dropTarget) {
    }
    
    @Override
    public void updateFocusableWindowState() {
    }
    
    @Override
    public void setZOrder(ComponentPeer componentPeer) {
    }
    
    private static native void initIDs();
    
    @Override
    public void applyShape(Region region) {
    }
    
    @Override
    public void setOpacity(float n) {
    }
    
    @Override
    public void setOpaque(boolean b) {
    }
    
    public void updateWindow(BufferedImage bufferedImage) {
    }
    
    @Override
    public void createScreenSurface(boolean b) {
    }
    
    @Override
    public void replaceSurfaceData() {
    }
    
    public boolean isMultipleMode() {
        return AWTAccessor.getFileDialogAccessor().isMultipleMode((FileDialog)this.target);
    }
    
    static void access$000(WFileDialogPeer wFileDialogPeer) {
        wFileDialogPeer._show();
    }
    
    static {
        initIDs();
        setFilterString((String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                try {
                    return ResourceBundle.getBundle("sun.awt.windows.awtLocalization").getString("allFiles");
                }
                catch (MissingResourceException ex) {
                    return "All Files";
                }
            }
        }));
    }
}
